@@ -99,19 +99,18 @@ define ([
             }
             return obj[prop];
         },
-        charsNeeded = function (arr) {
-            var len, needed;
+        maxLength = function (arr) {
+            var len, max;
             len = arr ? arr.length : 0;
             console.log("arr has " + len + " members");
-            needed = arr.reduce(function(prevVal, elem) {
+            max = arr.reduce(function(prevVal, elem) {
                 if (elem.text.length > prevVal) {
                     prevVal = elem.text.length;
                 }
                 return prevVal;
             }, arr[0].name.length);
-            needed += 2; // ': '
-            console.log("The longest entry needs " + needed + " characters total");
-            return needed;
+            console.log("The longest entry is " + max + " characters long");
+            return max;
         },
         browserNavMenu = function (len, pos) {
             var r = '',
@@ -235,9 +234,6 @@ define ([
             r += '</div>';
             return r;
         },
-        //
-        // dmenu target
-        //
         dmenu: function (dmn) {
             console.log("Entering html.dmenu with argument " + dmn);
             // dmn is dmenu name
@@ -266,9 +262,6 @@ define ([
             r += '</form>';
             return r;
         },
-        //
-        // dform target
-        //
         dform: function (dfn) {
             // dfn is dform name
             // dfo is dform object
@@ -293,7 +286,7 @@ define ([
                 // entry)
                 allEntries = lib.forceArray(dfo.entriesRead);
                 allEntries = allEntries.concat(dfo.entriesWrite);
-                needed = charsNeeded(allEntries);
+                needed = maxLength(allEntries);
 
                 // READ-ONLY entries first
                 len = dfo.entriesRead ? dfo.entriesRead.length : 0;
@@ -358,7 +351,7 @@ define ([
                 // determine characters needed for padding (based on longest
                 // entry)
                 allEntries = lib.forceArray(dbo.entries);
-                needed = charsNeeded(allEntries);
+                needed = maxLength(allEntries) + 2;
 
                 // display entries
                 len = dbo.entries ? dbo.entries.length : 0;
@@ -393,9 +386,6 @@ define ([
                 return r;
             };
         },
-        //
-        // dnotice target
-        //
         dnotice: function (dnn) {
             console.log("Entering html.dnotice with argument " + dnn);
             // dnn is dnotice name
@@ -411,6 +401,88 @@ define ([
                 r += '</div>';
                 return r;
             };
+        },
+        dtable: function (dtn) {
+            var dto = target.pull(dtn);
+            return function (set) {
+
+                console.log("Generating source code of dtable " + dtn);
+                console.log("dto", dto);
+                var r = '<form id="' + dto.name + '">',
+                    entry,
+                    column,
+                    row,
+                    col,
+                    maxl = [];
+
+                r += '<br><b>' + dto.title + '</b><br><br>';
+
+                if (dto.preamble) {
+                    r += dto.preamble + '<br><br>';
+                }
+
+                // populate maxl array
+                for (column = 0; column < dto.entries.length; column += 1) {
+                    console.log("Column " + column);
+                    entry = dto.entries[column];
+                    var elems = set.map(function (obj) {
+                        return obj[entry.prop];
+                    });
+                    var elemlengths = elems.map(function (elem) {
+                        var ep = ((elem === null) ? '' : elem).toString();
+                        return ep.length;
+                    });
+                    maxl[column] = elemlengths.reduce(function (a, b) {
+                        return (a > b) ? a : b;
+                    });
+                }
+
+                // display table header
+                for (column = 0; column < dto.entries.length; column += 1) {
+                    entry = dto.entries[column];
+                    if (lib.privCheck(entry.aclProfileRead)) {
+                        r += '<span style="text-decoration: underline">';
+                        r += lib.rightPadSpaces(entry.text, maxl[column]);
+                        r += '</span>';
+                    }
+                    if (column !== dto.entries.length - 1) {
+                        r += ' ';
+                    }
+                }
+                r += '<br>';
+
+                // display table rows
+                if (set.length > 0) {
+                    for (row = 0; row < set.length; row += 1) {
+                        var obj = set[row];
+                        for (column = 0; column < dto.entries.length; column += 1) {
+                            entry = dto.entries[column];
+                            console.log("entry", entry);
+                            if (lib.privCheck(entry.aclProfileRead)) {
+                                var val = obj[entry.prop];
+                                console.log("value", val);
+                                r += lib.rightPadSpaces(val, maxl[column]);
+                            }
+                            if (column !== dto.entries.length - 1) {
+                                r += ' ';
+                            }
+                        }
+                        r += '<br>';
+                    }
+                    r += '<br>';
+                }
+
+		// miniMenu at the bottom: selections are target names defined
+		// in the 'miniMenu' property of the dform object
+                r += miniMenu(dto.miniMenu);
+
+                // your choice section
+                r += yourChoice();
+
+                r += '</form>';
+                console.log("Assembled source code for " + dtn + " - it has " + r.length + " characters");
+                return r;
+            }
         }
     };
 });
