@@ -375,6 +375,33 @@ define ([
                 dnoticeSubmit(dno);
             });
             $('#' + dno.name).on("keypress", mmKeyListener);
+        },
+
+        //
+        // dtable handlers
+        // 
+        dtableState = {
+            "obj": null,
+            "set": null,
+        },
+        dtableSubmit = function (dto) {
+            target.pull(dto.miniMenu.back[1]).start();
+        },
+        dtableListen = function () {
+            var dto = dtableState.obj,
+                set = dtableState.set;
+            console.log("Listening in table " + dto.name);
+            $('#mainarea').html(dto.source(set));
+            lib.holdObject(set); // hold object so hooks can get it
+            $('#result').html("Displaying table with " + set.length + " objects");
+            $('#' + dto.name).submit(suppressSubmitEvent);
+            $('input[name="sel"]').val('').focus();
+            $('#submitButton').on("click", function (event) {
+                event.preventDefault;
+                console.log("Submitting table " + dto.name);
+                dtableSubmit(dto);
+            });
+            $('#' + dto.name).on("keypress", mmKeyListener);
         };
 
     return {
@@ -437,6 +464,31 @@ define ([
                 $('input[name="sel"]').focus();
                 dnoticeListen(dno);
             };
+        },
+        dtableListen: dtableListen, // export so other modules can call it
+        dtableState: dtableState, // export so other modules can modify browser state
+        dtable: function (dtn) {
+            var dto = target.pull(dtn);
+            if (dtn) {
+                // when called with dtn (dtable name) argument, we assume
+                // that we are being called from the second stage of dtable
+                // initialization (i.e., one-time event) -- generate and
+                // return the start function for this dtable
+                return function () {
+                    lib.clearResult();
+                    console.log('Starting new ' + dtn + ' dtable');
+                    // (re)initialize dtable state
+                    dtableState.obj = target.pull(dtn);
+                    dtableState.set = dto.hook();
+                    // start browsing
+                    dtableListen();
+                };
+            } else {
+                // when called _without_ an argument, we assume that there
+                // is an existing browser state to return to
+                console.log('Returning to previous ' + dtableState.obj.name + ' dtable state');
+                dtableListen();
+            }
         }
     }
 });
